import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;
import ij.measure.*;
import ij.plugin.MeasurementsWriter;
import java.util.*;

enum Side{
	MINX, MAXX, MINY, MAXY}

// Particles8_  by Gabriel Landini, G.Landini@bham.ac.uk
// Copyright (c) G. Landini
// http://www.mecourse.com/landinig/software/software.html
//
// Phytholiths is derived from the ImageJ plugin Particles8_ version 2.12 
// G. Landini does not maintain the Phytoliths plugin and is not responsible for its accuracy.
//
// 1/10/2014
// AnnaLisa Maddox Davis, phytolith morphometery analysis (based on G. Landini's Particles8 plugin) 2012-2013

public class Phytoliths_ implements PlugInFilter {
	protected String selectedOption;
	protected boolean doIwhite;
	protected boolean doIdeleteborders;
	protected boolean doIminmax;
	protected boolean doIshowstats;
	protected boolean doIlabel;
	protected boolean doImorpho;
	protected boolean doIoverwrite;
	protected int stk=0, mi, ma;
	protected String redirected;
	protected ResultsTable rt= ResultsTable.getResultsTable();

	protected ImageStack istk;
	protected ImagePlus imp2;
	protected ImageProcessor ip2;
	ImagePlus im = null;
	protected int bits;
	
	protected double unit_conversion;
	protected double test_cal;
	protected String units;
	protected List convP = new List();
	protected List perim = new List();
	protected int AD_pixel_count = 0;

	public int setup(String arg, ImagePlus imp) {
		
		int i=0, j=0;

		if (imp==null) {
			IJ.error("No image found.");
			return DONE;
		}
		int  thisimage=imp.getID();
		//ImagePlus imp = WindowManager.getCurrentImage();
		Calibration cal = imp.getCalibration();
		test_cal = cal.getX(1);
		if(cal.pixelWidth!=cal.pixelHeight){
			IJ.error("Image pixels are not squared.");
			return DONE;
		}

		ImageStatistics stats;
		stats=imp.getStatistics();
		if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount){
			IJ.error("8-bit binary image (0 and 255) required.");
			return DONE;
		}	

		istk = imp.getStack();
		int[] wList = WindowManager.getIDList();

		String[] titles = new String[wList.length];
		String none = "None";
		titles[0] = none;
		
		// avoid redirecting to itself	
		for (i=0; i<wList.length; i++) {
			if (wList[i]!=thisimage){
				j++;
				imp = WindowManager.getImage(wList[i]);//getImage
				titles[j] = imp!=null?imp.getTitle():"";
				//IJ.log(titles[j]);
			}
		}

		if (arg.equals("about"))
			{showAbout(); return DONE;}
		if (IJ.versionLessThan("1.42j")){
			IJ.error("ImageJ 1.42j or newer required.");
			return DONE;
		}

		GenericDialog gd = new GenericDialog("Phytoliths_", IJ.getInstance());
		gd.addMessage("Only \'square pixel\' images supported!");
//		gd.addCheckbox("White particles [255] on black [0] background",true);
//		gd.addCheckbox("Exclude edge particles",false);
//		gd.addCheckbox("Label Particles",false);
//		gd.addCheckbox("Morphology Parameters",false);
//		String [] DisplayOption={"Particles", "Coordinates", "CentreOfMass"};
//		gd.addChoice("Show", DisplayOption, DisplayOption[0]);
		gd.addCheckbox("Filter by size (pixels):",false);
		gd.addNumericField ("        Minimum size  ",  0, 0);
		gd.addNumericField ("        Maximum size ",  9999999, 0);
		gd.addCheckbox("Display results",true);
		gd.addCheckbox("Overwrite results",false);
		gd.addChoice("Redirect to", titles, titles[0]);
		gd.addNumericField("Pixels/Unit", 1, 0);
		gd.addStringField("Units", "Pixels", 3);
		gd.addMessage("Modifed version of G. Landini's Particles8_ v2.12\nby AnnaLisa Davis and Ben Draut");

		gd.showDialog();
		if (gd.wasCanceled())
			return DONE;
		
		doIwhite = true;
		doIdeleteborders=false;
		doIlabel=false;
		doImorpho=true;
		selectedOption = "Particles";
//		doIwhite = gd.getNextBoolean ();//first checkbox - white particles on black background
//		doIdeleteborders = gd.getNextBoolean ();//2nd checkbox - edge particles
//		doIlabel = gd.getNextBoolean ();//3rd checkbox - label particles
//		doImorpho = gd.getNextBoolean ();//4th checkbox - morphology parameters
		boolean doReset = doImorpho && rt.getCounter()>1 && rt.getColumnIndex("Roundness")<0; //wr  %who know what this line is supposed to be
//		selectedOption = gd.getNextChoice ();//1st choice selection - display option

		doIminmax = gd.getNextBoolean ();//5th checkbox - filter by size
		mi = (int)gd.getNextNumber();//1st numeric field - min size
		ma = (int)gd.getNextNumber();//2nd numeric field - max size
		doIshowstats = gd.getNextBoolean ();//6th checkbox - display results
		doIoverwrite = gd.getNextBoolean ();//7th checkbox - overwrite results
		if (doIoverwrite){ //wr %overwrite selected - present option to save previous results
			if (!IJ.macroRunning()){
				if (rt.getCounter()>0) {
					SaveChangesDialog di = new SaveChangesDialog(IJ.getInstance(), "Save "+rt.getCounter()+" measurements?");
					if (di.cancelPressed())
						return DONE;
					else if (di.savePressed())
						new MeasurementsWriter().run("");
				}
			}
			rt.reset();// clear results table
		}
		//rt.show("Results");

		redirected = gd.getNextChoice ();//2nd choice selection - redirect option
		double temp_conversion = gd.getNextNumber();//3rd numeric field - pixel/units ratio
		unit_conversion = 1/temp_conversion;//convert to units/pixel ratio
		units = gd.getNextString();//text field - units
		if (!redirected.equals("None")){
			imp2=WindowManager.getImage(redirected);
			ip2 = imp2.getProcessor();
			bits=imp2.getBitDepth();
			if(bits==32) {
				IJ.error("Cannot redirect to 32bit images.");
				return DONE;
			}
		}
		im = imp;
		return DOES_8G+DOES_STACKS;
	}

	public void run(ImageProcessor ip) {
		stk++;
		int xe = ip.getWidth();//image x boundary
		int ye = ip.getHeight();//image y boundary
		String slabel = istk.getSliceLabel(stk)+"\n"; //add a \n in case it is a null
		        if (slabel.length()>0) slabel=slabel.replaceAll("\\n",""); // remove "\n"
		int X = xe-1, Y = ye-1, XY = X*Y;
		int [] xd = {1,1,1,0,-1,-1,-1,0};
		int [] yd = {-1,0,1,1,1,0,-1,-1};
		int [] g = {6,0,0,2,2,4,4,6};
		double [] z = {1.414213538169861, 1.0, 1.414213538169861, 1.0, 1.414213538169861, 1.0, 1.414213538169861, 1.0};
		double [] chresults = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}; // convex hull, cx, cy, radius of minimal bounding circle, convex area, feret, fx1, fy1, fx2, fy2
		int x=0, y=0, x1, y1, q, uy, dy, rx, lx, pixs, fig, x2, y2, py=0, px=0, nn, newcol, f1, fx1=-1, fx2=-1, fy1=-1, fy2=-1, vl, pb1, pb2;
		long m10, m01;
		int pxgrey=0, pxred=0,  pxgreen=0, pxblue=0;
		double nz, narea, mcx, mcy, feret, tferet, rads=180/Math.PI, angle=0.0, breadth1, breadth2, minr, maxr;

		float [] parsR = new float [12];
		float [] parsG = new float [12];
		float [] parsB = new float [12];

		Vector vx = new Vector();
		Vector vy = new Vector();

		Vector vr = new Vector();
		Vector vg = new Vector();
		Vector vb = new Vector();


		if (ip2!=null)
			imp2.setSlice(stk); // advance if stack is being redirected

		FloodFiller ff = new FloodFiller(ip);

		// if not white particles then invert
		if (doIwhite==false){
			for(y=0;y<ye;y++) {
				for(x=0;x<xe;x++)
					ip.putPixel(x,y,255-ip.getPixel(x,y));
			}
		}

		if(doIdeleteborders){
			ip.setColor(0);
			for (y=0; y<ye; y++){
				if (ip.getPixel(0,y)==255) ff.fill8(0, y);
				if (ip.getPixel(X,y)==255) ff.fill8(X, y);
			}
			for (x=0; x<xe; x++){
				if (ip.getPixel(x,0)==255) ff.fill8(x, 0);
				if (ip.getPixel(x,Y)==255) ff.fill8(x, Y);
			}
		}

		ip.setColor(254);
		fig=0;
		// find first scanned pixel
		for (y=0; y<ye; y++)
		{
			convP.clear(); //convex perimeter list is empty for each particle
			perim.clear(); 
			AD_pixel_count=0;
			if (stk==1 && (y%25)==0) //wr
				IJ.showProgress(y, ye-1);
			for (x=0; x<xe; x++){
				if (ip.getPixel(x,y)==255)
				{
				
					vx.clear();
					vy.clear();
					vr.clear();
					vg.clear();
					vb.clear();

					vx.add(new Integer(x));
					vy.add(new Integer(y));
					perim.insertFront(x, y);
				
					x1=x;
					y1=y;
					//bounding box
					uy=y;//upper y
					dy=0;//lower y
					rx=0;//right x
					lx=xe;//left x

					nz=0;
					narea=0;
					pixs=0;
					//single pixel?
					if(ip.getPixel(x+1,y)+ip.getPixel(x,y+1)+ip.getPixel(x-1,y+1)+ip.getPixel(x+1,y+1)==0)
					{
						if (doIminmax && (mi>0)) {
							ip.putPixel(x,y,0);
						}
						else{
							ip.putPixel(x,y,((fig  % 251) + 1)); //new colour if labelling
							if (ip2!=null){
								pxgrey = ip2.getPixel(x,y);// look at px,py of redirected
								if (bits==24){
									pxred = (pxgrey & 0xff0000)>>16;
									pxgreen = (pxgrey & 0x00ff00)>>8;
									pxblue = (pxgrey & 0x0000ff);
								}
							}
							pixs=1;//don't actally analyze single pixel data
							fig++;

//							rt.incrementCounter();
//							rt. addValue("Area "+units+"^2", pixs*unit_conversion*unit_conversion);
//							rt. addValue("Convex Area "+units+"^2", 1);
//							rt. addValue("Perimeter "+units, 2);
//							rt. addValue("Convex Perimeter "+units, 2);
//							rt. addValue("Length (Feret) "+units, 1.414);
//							rt. addValue("Breadth "+units, 1);
//							rt. addValue("Fiber Length "+units, 1);
//							rt. addValue("Width "+units, 1);
//							rt. addValue("Equivalent Diameter (ArEquivD) "+units, 1);
//							rt. addValue("Inscribed Radius (MinR) "+units, .5);
//							rt. addValue("Form Factor (Circ)", -1);
//							rt. addValue("Roundness", -1);
//							rt. addValue("Convexity", -1);
//							rt. addValue("Solidity", -1);
//							rt. addValue("Compactness", -1);
//							rt. addValue("Aspect Ratio",-1);
//							rt. addValue("Elongation", -1);
//							rt. addValue("Curl", -1);
//
//							if (ip2!=null){
//								if (bits==24){
//									rt. addValue("RedIntDen", pxred);
//									rt. addValue("RedMin", pxred);
//									rt. addValue("RedMax", pxred);
//									rt. addValue("RedMode", pxred);
//									rt. addValue("RedMedian", pxred);
//									rt. addValue("RedAverage",(float) pxred);
//									rt. addValue("RedAvDev", (float) 0);
//									rt. addValue("RedStDev", (float) 0);
//									rt. addValue("RedVar", (float) 0);
//									rt. addValue("RedSkew", (float) 0);
//									rt. addValue("RedKurt", (float) 0);
//									rt. addValue("RedEntr", (float) 0);
//
//									rt. addValue("GreenIntDen", pxgreen);
//									rt. addValue("GreenMin", pxgreen);
//									rt. addValue("GreenMax", pxgreen);
//									rt. addValue("GreenMode", pxgreen);
//									rt. addValue("GreenMedian", pxgreen);
//									rt. addValue("GreenAverage",(float) pxgreen);
//									rt. addValue("GreenAvDev", (float) 0);
//									rt. addValue("GreenStDev", (float) 0);
//									rt. addValue("GreenVar", (float) 0);
//									rt. addValue("GreenSkew", (float) 0);
//									rt. addValue("GreenKurt", (float) 0);
//									rt. addValue("GreenEntr", (float) 0);
//
//									rt. addValue("BlueIntDen", pxblue);
//									rt. addValue("BlueMin", pxblue);
//									rt. addValue("BlueMax", pxblue);
//									rt. addValue("BlueMode", pxblue);
//									rt. addValue("BlueMedian", pxblue);
//									rt. addValue("BlueAverage",(float) pxblue);
//									rt. addValue("BlueAvDev", (float) 0);
//									rt. addValue("BlueStDev", (float) 0);
//									rt. addValue("BlueVar", (float) 0);
//									rt. addValue("BlueSkew", (float) 0);
//									rt. addValue("BlueKurt", (float) 0);
//									rt. addValue("BlueEntr", (float) 0);
//								}
//								else if (bits==8 || bits==16){
//									rt. addValue("GrIntDen", pxgrey);
//									rt. addValue("GrMin", pxgrey);
//									rt. addValue("GrMax", pxgrey);
//									rt. addValue("GrMode", pxgrey);
//									rt. addValue("GrMedian", pxgrey);
//									rt. addValue("GrAverage",(float) pxgrey);
//									rt. addValue("GrAvDev", (float) 0);
//									rt. addValue("GrStDev", (float) 0);
//									rt. addValue("GrVar", (float) 0);
//									rt. addValue("GrSkew", (float) 0);
//									rt. addValue("GrKurt", (float) 0);
//									rt. addValue("GrEntr", (float) 0);
//								}
//							}
//							rt.addResults(); //wr gl
						}
					}
					else { //go around the edge
						x1=x;
						y1=y;
						ip.putPixel(x,y,254);
						q=0;
						while(true){
							if (ip.getPixel(x1+xd[q],y1+yd[q])>0){
								x2=x1;
								y2=y1;
								x1+=xd[q];
								y1+=yd[q];
								narea+=(double)((y1+y2)*(x2-x1))*.5;
								ip.putPixel(x1,y1,254);
								//update coordinates of ROI box
								if(x1 > rx) rx = x1;
								if(x1 < lx) lx = x1;
								if(y1 > dy) dy = y1;
								if(y1 < uy) uy = y1;
								nz+=z[q]; //perimeter
								q=g[q];//directs to next pixel to analyze

								//check for *that* unique pixel configuration
								if (x1==x){
									if (y1==y){
										if (ip.getPixel(x-1,y+1)!=255){
											break;
										}
									}
								}
								vx.add(new Integer(x1));
								vy.add(new Integer(y1));
								perim.insertFront(x1, y1);
								if(q==0)
									perim.start().Next.cc = 7;
								else
									perim.start().Next.cc = q-1;
							}
							else
								q=(q+1)%8;
						}
						//System.out.println("Debug:"+xe+" "+ye+":   "+ uy+ " "+lx+" "+dy+" "+rx);

						vl=vx.size();
						// Convex hull here
						convh( vx, vy, vl, chresults);
						//IJ.log(""+results[0]+" "+results[1]+" "+results[2]+" "+results[3]);
						
						// feret diameter here
						feret=chresults[5]+1;//+1 accounts for pixel edges
						fx1=(int)chresults[6];
						fy1=(int)chresults[7];
						fx2=(int)chresults[8];
						fy2=(int)chresults[9];

						//IF f2x - fx = 0 THEN THETA = 90 ELSE THETA = -INT(ATN((f2y - fy) / (f2x - fx)) * 57.29583)
						if (fx2==fx1)
							angle=90.0;
						else
							angle=-Math.atan((double)(fy2-fy1)/(double)(fx2-fx1))*rads;
						if (angle<0)
							angle=180+angle;

						// breadth here	
						breadth1=0;
						breadth2=0;
						pb1=0;
						pb2=0;
						for (f1=0;f1<vl;f1++){
							tferet=((fy1-fy2)*((Integer)(vx.get(f1))).intValue() +(fx2-fx1)*((Integer)(vy.get(f1))).intValue()+(fx1*fy2 -fx2*fy1))/feret;
							if (tferet>breadth1){
								breadth1=tferet;
								pb1=f1;
							}
							if (tferet<breadth2){
								breadth2=tferet;
								pb2=f1;
							}
						}
						//IJ.log("Breadth1: "+breadth1+"   "+((Integer)(vx.get(pb1))).intValue()+" "+((Integer)(vy.get(pb1))).intValue()+" breadth2: "+breadth2+"   "+((Integer)(vx.get(pb2))).intValue()+" "+((Integer)(vy.get(pb2))).intValue());

						//scan bounding box from top left to bottom right and {flood) with 254 if pixel is 255 and any neighbour pixel is 254
						for(py=uy;py<dy+1;py++) {//+1 on py; -1 on dy
							for(px=lx;px<rx+1;px++) {//+1 on px; -1 on rx
								if(ip.getPixel(px,py)!=0)
									AD_pixel_count++;
								if (ip.getPixel(px,py)==255){
									if(ip.getPixel(px-1,py-1)==254 ||
									   ip.getPixel(px,  py-1)==254 ||
									   ip.getPixel(px+1,py-1)==254 ||
									   ip.getPixel(px-1,py  )==254 ||
									   ip.getPixel(px+1,py  )==254 ||
									   ip.getPixel(px+1,py+1)==254 ||
									   ip.getPixel(px,  py+1)==254 ||
									   ip.getPixel(px-1,py+1)==254)
										ff.fill8(px, py);
								}
							}
						}

						newcol = (fig  % 251) + 1; //new colour if labelling
						m10 = 0;
						m01 = 0;
						mcx=0.0;
						mcy=0.0;
						if(doIminmax) //new test
							nn = 253; //must have an extra colour not used elsewhere
						 else
						 	nn = newcol;

						//momentum %also counts pixels in image
						for(py=uy;py<=dy;py++)
						{
							for(px=lx;px<=rx;px++){
								if(ip.getPixel(px,py)== 254){
									pixs+=1;
									m10+=px;
									m01+=py;
									ip.putPixel(px,py,nn); //label particle either newcol or 253 (to be tested for deletion).
									if (ip2!=null){
										if (bits==24){
											pxgrey=ip2.getPixel(px,py);
											vr.add(new Integer(((pxgrey & 0xff0000)>>16)));
											vg.add(new Integer(((pxgrey & 0x00ff00)>>8)));
											vb.add(new Integer((pxgrey & 0x0000ff)));
										}
										else
											vr.add(new Integer(ip2.getPixel(px,py)));// look at px,py of original to get greys, put into vector 
									}
								}
							}
						}
						if (ip2!=null){
							if (bits==24){
								stats(pixs, vr, parsR);
								stats(pixs, vg, parsG);
								stats(pixs, vb, parsB);
							}
							else if (bits==8 || bits==16){
								stats(pixs, vr, parsR);
								 //integral, min, max, mode, median, ave, avedev, sdev, var, skew, kurt, entropy
							}
						}

						mcx = ((double)m10 / (double)pixs);
						mcy = ((double)m01 / (double)pixs);
						
						//Minimal Maximal radii
						minr=Double.MAX_VALUE; //100000000000000.0;
						maxr=-1;
						for (f1=0;f1<vl;f1++){
							tferet = Math.pow(Math.abs(mcx - ((Integer)(vx.get(f1))).intValue()) + 0.5, 2) + 
								     Math.pow(Math.abs(mcy - ((Integer)(vy.get(f1))).intValue()) + 0.5, 2);
							if (tferet>maxr){
								maxr=tferet;
							}
							if (tferet<minr){
								minr=tferet;
							}
						}
						minr=Math.sqrt(minr);
						maxr=Math.sqrt(maxr);
						//IJ.log("MinR: "+minr+"   MaxR:"+maxr);
						
						//connect perimeter chain code ends
						Node s = perim.start();
						Node l = perim.last();
						if(s.x<l.x)
							if(s.y<l.y)
								perim.last().cc = 3;
							else if(s.y==l.y)
								perim.last().cc=4;
							else
								perim.last().cc = 5;
						else if(s.x==l.x)
							if(s.y<l.y)
								perim.last().cc = 2;
							else
								perim.last().cc = 6;
						else
							if(s.y<l.y)
								perim.last().cc= 1;
							else if(s.y==l.y)
								perim.last().cc = 0;
							else
								perim.last().cc = 7;
						//find boundary on perim chain code
						double cc_perim = 0;
						Node current = perim.start();
						for(int w = 0; w<perim.size; w++)
						{
							cc_perim += 1+current.cc%2*.414;//add 1 if even 1.414 if odd
							current=current.Next;
						}
						
						if (doIminmax){ //is colour 253
							if (pixs<mi || pixs>ma){ // erase (filter) particles
								for(py=uy;py<=dy;py++){
									for(px=lx;px<=rx;px++){
										if(ip.getPixel(px,py)== nn) //nn=253, delete it
											ip.putPixel(px,py,0);
									}
								}
							}
							else{ //particle not to be deleted...
								if (selectedOption.equals("Coordinates") || selectedOption.equals("CentreOfMass")){ 
									ip.setColor(0); //erase it
									ff.fill8(x, y);
									ip.setColor(254);
								}
								else{ //change colour from 253 to newcol
									for(py=uy;py<=dy;py++){
										for(px=lx;px<=rx;px++){
											if(ip.getPixel(px,py)== nn) //253
												ip.putPixel(px,py,newcol);
										}
									}
								}
								fig++;

								double[] b_l = Breadth(vx,vy);
								float breadth = (float)b_l[0];
								float width = breadth;
								float length = (float)b_l[1];
								rt.incrementCounter();
								Conv_Perim(vx,vy);
								float cp = (float)chresults[0];
								float ca = (float)Conv_Area();
								float area = pixs;
								if(ca==-1)
									ca=area;
								rt. addValue("Area "+units+"^2", pixs*unit_conversion*unit_conversion);
								rt. addValue("Convex Area "+units+"^2",ca*unit_conversion*unit_conversion);
								float perimeter = (float)nz;
								rt. addValue("Perimeter "+units, perimeter*unit_conversion);
								rt. addValue("Convex Perimeter "+units, cp*unit_conversion);
								rt. addValue("Length (Feret) "+units, length*unit_conversion);
//								rt. addValue("Breadth "+units, breadth*unit_conversion);
								float fiber_length = (float)IntelligentFiberLength(ip, vx, vy, length, width, perimeter, area);
								rt. addValue("Fiber Length "+units, fiber_length*unit_conversion);
								rt. addValue("Width "+units, width*unit_conversion);
								rt. addValue("Equivalent Diameter (ArEquivD) "+units, (float) Math.sqrt((4.0/Math.PI)*pixs)*unit_conversion);
								rt. addValue("Inscribed Radius (MinR) "+units, (float)minr*unit_conversion);
								rt. addValue("Form Factor (Circ)", (float) (4.0*Math.PI*area/Math.pow(perimeter,2)));
								rt. addValue("Roundness", (float) (4.0*area/(Math.PI*Math.pow(feret,2))));
								rt. addValue("Convexity", (float) (cp/perimeter));
								rt. addValue("Solidity", ((ca>0)?(float)(area/ca):-1));
								rt. addValue("Compactness", (float) (Math.sqrt((4.0/Math.PI)*area)/feret));
								rt. addValue("Aspect Ratio",  length/width);
								rt. addValue("Elongation", fiber_length/width);
								rt. addValue("Curl", length/fiber_length);
									
								if (ip2!=null){
									if (bits==24){
										rt. addValue("RedIntDen", (int) parsR[0]);
										rt. addValue("RedMin",  (int)parsR[1]);
										rt. addValue("RedMax", (int)parsR[2]);
										rt. addValue("RedMode", (int)parsR[3]);
										rt. addValue("RedMedian", (int)parsR[4]);
										rt. addValue("RedAverage", parsR[5]);
										rt. addValue("RedAvDev", parsR[6]);
										rt. addValue("RedStDev", parsR[7]);
										rt. addValue("RedVar", parsR[8]);
										rt. addValue("RedSkew", parsR[9]);
										rt. addValue("RedKurt", parsR[10]);
										rt. addValue("RedEntr", parsR[11]);

										rt. addValue("GreenIntDen", (int)parsG[0]);
										rt. addValue("GreenMin", (int)parsG[1]);
										rt. addValue("GreenMax", (int)parsG[2]);
										rt. addValue("GreenMode", (int)parsG[3]);
										rt. addValue("GreenMedian", (int)parsG[4]);
										rt. addValue("GreenAverage", parsG[5]);
										rt. addValue("GreenAvDev", parsG[6]);
										rt. addValue("GreenStDev", parsG[7]);
										rt. addValue("GreenVar", parsG[8]);
										rt. addValue("GreenSkew", parsG[9]);
										rt. addValue("GreenKurt", parsG[10]);
										rt. addValue("GreenEntr", parsG[11]);

										rt. addValue("BlueIntDen", (int)parsB[0]);
										rt. addValue("BlueMin", (int)parsB[1]);
										rt. addValue("BlueMax", (int)parsB[2]);
										rt. addValue("BlueMode", (int)parsB[3]);
										rt. addValue("BlueMedian", (int)parsB[4]);
										rt. addValue("BlueAverage", parsB[5]);
										rt. addValue("BlueAvDev", parsB[6]);
										rt. addValue("BlueStDev", parsB[7]);
										rt. addValue("BlueVar", parsB[8]);
										rt. addValue("BlueSkew", parsB[9]);
										rt. addValue("BlueKurt", parsB[10]);
										rt. addValue("BlueEntr", parsB[11]);
									}
									else if (bits==8 || bits==16){
										rt. addValue("GrIntDen", (int) parsR[0]);
										rt. addValue("GrMin", (int)parsR[1]);
										rt. addValue("GrMax", (int)parsR[2]);
										rt. addValue("GrMode", (int)parsR[3]);
										rt. addValue("GrMedian", (int)parsR[4]);
										rt. addValue("GrAverage", parsR[5]);
										rt. addValue("GrAvDev", parsR[6]);
										rt. addValue("GrStDev", parsR[7]);
										rt. addValue("GrVar", parsR[8]);
										rt. addValue("GrSkew", parsR[9]);
										rt. addValue("GrKurt", parsR[10]);
										rt. addValue("GrEntr", parsR[11]);
									} //integral, min, max, mode, median, ave, avedev, sdev, var, skew, kurt
								}
								rt.addResults(); //wr gl
							}
						}
						else{ //particle is already the correct colour
							if (selectedOption.equals("Coordinates") || selectedOption.equals("CentreOfMass" )){
								ip.setColor(0); //erase it
								ff.fill8(x, y);
								ip.setColor(254);
							}
							fig++;
							double[] b_l = Breadth(vx,vy);
							float breadth = (float)b_l[0];
							float width = breadth;
							float length = (float)b_l[1];
							rt.incrementCounter();
							Conv_Perim(vx,vy);
							float cp = (float)chresults[0];
							float ca = (float)Conv_Area();
							float area = pixs;
							if(ca==-1)
								ca=area;
							rt. addValue("Area "+units+"^2", pixs*unit_conversion*unit_conversion);
							rt. addValue("Convex Area "+units+"^2",ca*unit_conversion*unit_conversion);
							float perimeter = (float)nz;
							rt. addValue("Perimeter "+units, (float)nz*unit_conversion);
							rt. addValue("Convex Perimeter "+units, cp*unit_conversion);
							rt. addValue("Length (Feret) "+units, length*unit_conversion);
//							rt. addValue("Breadth "+units, breadth*unit_conversion);
							float fiber_length = (float)IntelligentFiberLength(ip, vx, vy, length, width, perimeter, area);
							rt. addValue("Fiber Length "+units, fiber_length*unit_conversion);
							rt. addValue("Width "+units, width*unit_conversion);
							rt. addValue("Equivalent Diameter (ArEquivD) "+units, (float) Math.sqrt((4.0/Math.PI)*area)*unit_conversion);
							rt. addValue("Inscribed Radius (MinR) "+units, (float)minr*unit_conversion);
							rt. addValue("Form Factor (Circ)", (float) (4.0*Math.PI*area/Math.pow(nz,2)));
							rt. addValue("Roundness", (float) (4.0*area/(Math.PI*Math.pow(feret,2))));
							rt. addValue("Convexity", (float) (cp/perimeter));
							rt. addValue("Solidity", ((ca>0)?(float)(area/ca):-1));
							rt. addValue("Compactness", (float) (Math.sqrt((4.0/Math.PI)*area)/feret));
							rt. addValue("Aspect Ratio", length/width);
							rt. addValue("Elongation", fiber_length/width);
							rt. addValue("Curl", length/fiber_length);
								
							if (ip2!=null){
								if (bits==24){
									rt. addValue("RedIntDen", (int) parsR[0]);
									rt. addValue("RedMin",  (int)parsR[1]);
									rt. addValue("RedMax", (int)parsR[2]);
									rt. addValue("RedMode", (int)parsR[3]);
									rt. addValue("RedMedian", (int)parsR[4]);
									rt. addValue("RedAverage", parsR[5]);
									rt. addValue("RedAvDev", parsR[6]);
									rt. addValue("RedStDev", parsR[7]);
									rt. addValue("RedVar", parsR[8]);
									rt. addValue("RedSkew", parsR[9]);
									rt. addValue("RedKurt", parsR[10]);
									rt. addValue("RedEntr", parsR[11]);

									rt. addValue("GreenIntDen", (int)parsG[0]);
									rt. addValue("GreenMin", (int)parsG[1]);
									rt. addValue("GreenMax", (int)parsG[2]);
									rt. addValue("GreenMode", (int)parsG[3]);
									rt. addValue("GreenMedian", (int)parsG[4]);
									rt. addValue("GreenAverage", parsG[5]);
									rt. addValue("GreenAvDev", parsG[6]);
									rt. addValue("GreenStDev", parsG[7]);
									rt. addValue("GreenVar", parsG[8]);
									rt. addValue("GreenSkew", parsG[9]);
									rt. addValue("GreenKurt", parsG[10]);
									rt. addValue("GreenEntr", parsG[11]);

									rt. addValue("BlueIntDen", (int)parsB[0]);
									rt. addValue("BlueMin", (int)parsB[1]);
									rt. addValue("BlueMax", (int)parsB[2]);
									rt. addValue("BlueMode", (int)parsB[3]);
									rt. addValue("BlueMedian", (int)parsB[4]);
									rt. addValue("BlueAverage", parsB[5]);
									rt. addValue("BlueAvDev", parsB[6]);
									rt. addValue("BlueStDev", parsB[7]);
									rt. addValue("BlueVar", parsB[8]);
									rt. addValue("BlueSkew", parsB[9]);
									rt. addValue("BlueKurt", parsB[10]);
									rt. addValue("BlueEntr", parsB[11]);
								}
								else if (bits==8 || bits==16){
									rt. addValue("GrIntDen", (int) parsR[0]);
									rt. addValue("GrMin", (int)parsR[1]);
									rt. addValue("GrMax", (int)parsR[2]);
									rt. addValue("GrMode", (int)parsR[3]);
									rt. addValue("GrMedian", (int)parsR[4]);
									rt. addValue("GrAverage", parsR[5]);
									rt. addValue("GrAvDev", parsR[6]);
									rt. addValue("GrStDev", parsR[7]);
									rt. addValue("GrVar", parsR[8]);
									rt. addValue("GrSkew", parsR[9]);
									rt. addValue("GrKurt", parsR[10]);
									rt. addValue("GrEntr", parsR[11]);
								}
							}
							rt.addResults(); //wr gl
						}
					}//end of block - going around edge
				}
			}
		}//end for loop finding pixels

		//if not labelling, change the colour back to 255
		if (doIlabel==false){
			for(y=0;y<ye;y++) {
				for(x=0;x<xe;x++){
					if (ip.getPixel(x,y)>0)
						ip.putPixel(x,y,255);
				}
			}
			//since the centre of mass is done at the end, coordinates are done here too
			if(selectedOption.equals("Coordinates")){
				for (x=rt.getCounter()-fig;x<rt.getCounter();x++)
					ip.putPixel((int)rt.getValue("XStart",x),(int)rt.getValue("YStart",x),255);//.. put startPixel
			}

			// do the centre of mass at the end because it may fall inside some other particle and that locks the labelling
			if(selectedOption.equals("CentreOfMass")){
				for (x=rt.getCounter()-fig;x<rt.getCounter();x++)
					ip.putPixel((int)Math.floor(rt.getValue("XM",x)+0.5),(int)Math.floor(rt.getValue("YM",x)+0.5),255);//.. put centre of mass
			}

			// if white particles, invert
			if (doIwhite==false){
				for(y=0;y<ye;y++) {
					for(x=0;x<xe;x++)
						ip.putPixel(x,y,255-ip.getPixel(x,y));
				}
			}
		}
		else{
			if(selectedOption.equals("Coordinates")){
				y=0;
				for (x=rt.getCounter()-fig;x<rt.getCounter();x++)
					ip.putPixel((int)rt.getValue("XStart",x),(int)rt.getValue("YStart",x),(y++  % 251) + 1);//.. put startPixel
			}

			if(selectedOption.equals("CentreOfMass")){
				y=0;
				for (x=rt.getCounter()-fig;x<rt.getCounter();x++)
					ip.putPixel((int)Math.floor(rt.getValue("XM",x)+0.5),(int)Math.floor(rt.getValue("YM",x)+0.5),(y++  % 251) + 1);//.. put centre of mass
			}

			if (doIwhite==false){
				for(y=0;y<ye;y++) {
					for(x=0;x<xe;x++){
						if (ip.getPixel(x,y)==0)
							ip.putPixel(x,y,255); //white background
					}
				}
			}
		}
		if (doIshowstats) rt.updateResults(); //wr
		IJ.showProgress(1.0);
		rt.show("Results");
	}

	void convh(Vector vx, Vector vy, int vl, double [] results) {
		//results[] = convex hull, cx, cy, radius of minimal bounding circle, convex area, feret, fx1, fy1, fx2, fy2
		
		int i, j, k=0, m;
		
		int[] x = new int[vl+1];
		int[] y = new int[vl+1];

		results[0]=0.0;//convex hull
		results[1]=0.0;//cx
		results[2]=0.0;//cy
		results[3]=0.0;//minR
		results[4]=0.0;//convex area
		
		for (j=0;j<vl;j++){
			x[j] = ((Integer)(vx.get(j))).intValue();
			y[j] = ((Integer)(vy.get(j))).intValue();
		}
		//x[vl]=x[0];
		//y[vl]=y[0];
		
		//cnvx hull
		int n=vl, min = 0, ney=0, h, h2, dx, dy, temp,ax, ay;
		double minangle, th, t, v, zxmi=0;

		for (i=1;i<n;i++){
			if (y[i] < y[min]) 
				min = i;
		}

		temp = x[0]; x[0] = x[min]; x[min] = temp;
		temp = y[0]; y[0] = y[min]; y[min] = temp;
		min = 0;

		for (i=1;i<n;i++){
			if (y[i] == y[0]){
				ney ++;
				if (x[i] < x[min]) min = i;
			}
		}
		temp = x[0]; x[0] = x[min]; x[min] = temp;
		temp = y[0]; y[0] = y[min]; y[min] = temp;

		min = 0;
		m = -1; 
		x[n] = x[min];
		y[n] = y[min];
		if (ney > 0) 
			minangle = -1;
		else
			minangle = 0;

		while (min != n ){
			m = m + 1;
			temp = x[m]; x[m] = x[min]; x[min] = temp;
			temp = y[m]; y[m] = y[min]; y[min] = temp;

			min = n ; //+1
			v = minangle;
			minangle = 360.0;
			h2 = 0;

			for (i = m + 1;i<n+1;i++){
				dx = x[i] - x[m];
				ax = Math.abs(dx);
				dy = y[i] - y[m];
				ay = Math.abs(dy);
  
				if (dx == 0 && dy == 0) 
					t = 0.0;
				else 
					t = (double)dy / (double)(ax + ay);
  
				if (dx < 0)
					t = 2.0 - t;
				else {
					if (dy < 0)
						t = 4.0 + t;
				}
				th = t * 90.0;

				if(th > v){
					if(th < minangle){
						min = i;
						minangle = th;
						h2 = dx * dx + dy * dy;
					}
					else{
						if (th == minangle){
							h = dx * dx + dy * dy;
							if (h > h2){
								min = i;
								h2 = h;
							}
						}
					}
				}
			}
			zxmi += Math.sqrt(h2);
		}
		m++;

		int[] hx = new int[m];// ROI polygon array 
		int[] hy = new int[m];

		for (i=0;i<m;i++){
			hx[i] =  x[i]; // copy to new polygon array
			hy[i] =  y[i];
			//IJ.log(" "+hx[i]+" "+hy[i]);
			if (i>0)
				results[4] += (double)((hy[i]+hy[i-1])*(hx[i-1]-hx[i]))*.5;
		}
		results[4] += (double)((hy[i-1]+hy[0])*(hx[i-1]-hx[0]))*.5;//convex area

		results[0]=(float)zxmi;//convex hull
		// get the edges between points
		double [] d = new double [(m *(m-1))/2]; // edge lenght
		int[] p1 = new int [(m *(m-1))/2]; // point 1
		int[] p2 = new int [(m *(m-1))/2]; // point 2

		double feret=-1;
		int pferet1=-1, pferet2=-1;
		k=0;
		for (i=0;i<m-1;i++){
			for (j=i+1;j<m;j++){
				d[k]= Math.sqrt(Math.pow(hx[i]-hx[j], 2.0) + Math.pow(hy[i]-hy[j], 2.0));
				if (d[k]>feret) {feret = d[k]; pferet1=i; pferet2=j;}
				p1[k]=i;
				p2[k]=j;
				k++;
			}
		}
		k--;

		results[5]=feret;//feret
		results[6]=hx[pferet1];//fx1
		results[7]=hy[pferet1];//fy1
		results[8]=hx[pferet2];//fx2
		results[9]=hy[pferet2];//fy2

		//sort distances
		boolean sw = true;
		double tempd;
		int p3;
		double tt, tttemp, radius, cx, cy;
		while (sw){
			sw=false;
			for(i=0;i<k-1;i++){
				if (d[i]<d[i+1]){
					tempd = d[i]; d[i] = d[i+1]; d[i+1] = tempd;
					temp = p1[i]; p1[i] = p1[i+1]; p1[i+1] = temp;
					temp = p2[i]; p2[i] = p2[i+1]; p2[i+1] = temp;
					sw = true;
				}
			}
		}

		//IJ.log("1:"+d[0]+" "+p1[0]+" "+p2[0]);
		//IJ.log("2:"+d[1]+" "+p1[1]+" "+p2[1]);
		radius=d[0]/2.0;

		cx=(hx[p1[0]]+hx[p2[0]])/2.0;
		cy=(hy[p1[0]]+hy[p2[0]])/2.0;

		// find largest distance from point c
		//sw=false;
		p3=-1;
		tt=radius;
		for (i=0;i<m;i++){
			tttemp=Math.sqrt(Math.pow(hx[i]-cx, 2.0) + Math.pow(hy[i]-cy, 2.0));
			if(tttemp>tt){
				tt=tttemp;
				//IJ.log("Largest from c: "+Math.sqrt(Math.pow(hx[i]-cx, 2.0) + Math.pow(hy[i]-cy, 2.0))+"  "+radius);
				p3=i;
			}
		}

		if (p3>-1){
			// 3 or more- points circle
			//IJ.log("p3:"+p3);
			//IJ.log("osculating circle p1, p2, p3");
			double [] op1 = new double [2];
			double [] op2 = new double [2];
			double [] op3 = new double [2];
			double [] circ = new double [3];
			double tD=Double.MAX_VALUE;
			int tp1=0, tp2=0, tp3=0, z;

			// GL new test: find the smallest osculating circle that contains all convex hull points
			for (i=0; i<m-2; i++){
				for (j=i+1; j<m-1; j++){
					for (k=j+1; k<m; k++){
						op1[0]=hx[i];
						op1[1]=hy[i];
						op2[0]=hx[j];
						op2[1]=hy[j];
						op3[0]=hx[k];
						op3[1]=hy[k];
						osculating(op1, op2, op3, circ);
						// IJ.log(""+i+ " "+j+" "+k+"   "+circ[2]);
						// store a large dummy radius
						if (circ[2]>0){
							sw=true;
							for (z=0;z<m;z++){
								tttemp=(float)Math.sqrt(Math.pow(hx[z]-circ[0], 2.0) + Math.pow(hy[z]-circ[1], 2.0));
								if(tttemp>circ[2]){
									sw=false; // points are outside it
									break; // don't check any more points
								}
							}
							if(sw){ //no CH points outside
								// store radius & coordinates
								// IJ.log(""+i+ " "+j+" "+k+"   "+circ[2]);
								if (circ[2]<tD){
									tp1=i;
									tp2=j;
									tp3=k;
									tD=circ[2];
								}
							}
						}
					}
				}
			}
			op1[0]=hx[tp1];
			op1[1]=hy[tp1];
			op2[0]=hx[tp2];
			op2[1]=hy[tp2];
			op3[0]=hx[tp3];
			op3[1]=hy[tp3];
			// IJ.log("Solved for:"+tp1+ " "+tp2+" "+tp3);
			osculating(op1, op2, op3, circ);
			radius=circ[2];
			//==========================
			// op1[0]=(double)hx[p1[0]];
			// op1[1]=(double)hy[p1[0]];
			// op2[0]=(double)hx[p2[0]];
			// op2[1]=(double)hy[p2[0]];
			// op3[0]=(double)hx[p3];
			// op3[1]=(double)hy[p3];
			//
			// osculating(op1, op2, op3, circ);
			// radius=circ[2];
			// IJ.log("Bounding circle (3 points) x: "+((float)circ[0])+", y: "+((float)circ[1])+", radius: "+ radius);
			results[1]=circ[0];
			results[2]=circ[1];
		}
		else{
			//2-point circle centred at cx, cy radius
			//IJ.log("Bounding circle (2 points) x: "+cx+", y: "+cy+", radius: "+ radius);
			results[1]=cx;
			results[2]=cy;
		}
		results[3]=radius;
	}//end convh

	
	void osculating( double [] pa, double [] pb, double [] pc, double [] centrad){
		
//		System.out.println("OSCULATING FUNCTION");
//		System.out.println("PA: [" + pa[0] + ", " + pa[1] + "]");
//		System.out.println("PB: [" + pb[0] + ", " + pb[1] + "]");
//		System.out.println("PC: [" + pc[0] + ", " + pc[1] + "]");
//		System.out.println("Centrad: " + centrad);
		
		// returns 3 double values: the centre (x,y) coordinates & radius
		// of the circle passing through 3 points pa, pb and pc
		double a, b, c, d, e, f, g;
		if ((pa[0]==pb[0] && pb[0]==pc[0]) || (pa[1]==pb[1] && pb[1]==pc[1])){ //colinear coordinates
			centrad[0]=-1; //x
			centrad[1]=-1; //y
			centrad[2]=-1; //radius
			return;
		}

		a = pb[0] - pa[0];
		b = pb[1] - pa[1];
		c = pc[0] - pa[0];
		d = pc[1] - pa[1];

		e = a*(pa[0] + pb[0]) + b*(pa[1] + pb[1]);
		f = c*(pa[0] + pc[0]) + d*(pa[1] + pc[1]);

		g = 2.0 * ( a * (pc[1] - pb[1]) - b * (pc[0] - pb[0]));
		// If g is 0 then the three points are colinear and no finite-radius
		// circle through them exists. Return -1 for the radius. Somehow this does not
		// work as it should (representation of double number?), so it is trapped earlier.
		centrad[0] = (d * e - b * f) / g;
		centrad[1] = (a * f - c * e) / g;
		centrad[2] =(float) Math.sqrt(Math.pow((pa[0] - centrad[0]),2) + Math.pow((pa[1] - centrad[1]),2));
	}


	void stats(int nc, Vector vg, float [] pars){
		// integral, min, max, mode, median, average, avg.deviation, std.deviation, variance, skewness, kurtosis, entropy
		int[] data = new int[nc];
		int i;
		float s = 0, min = Float.MAX_VALUE, max = -Float.MAX_VALUE, totl=0, ave=0, adev=0, sdev=0, var=0, skew=0, kurt=0, mode=0, median=0, p, q, msf=0, nco2 = (float)nc/2;
		double log2= Math.log(2), pr;
		for(i=0;i<nc;i++){
			data[i]= ((Integer)(vg.get(i))).intValue();
			totl+= data[i];
			if(data[i]<min) min = data[i];
			if(data[i]>max) max = data[i];
		}

		ave = totl/nc;

		for(i=0;i<nc;i++){
			s = data[i] - ave;
			adev+=Math.abs(s);
			p = s * s;
			var+= p;
			p*=s;
			skew+= p;
			p*= s;
			kurt+= p;
		}

		adev/= nc;
		var/=nc-1;
		sdev = (float) Math.sqrt(var);

		if(var> 0){
			skew = (float)skew / (nc * (float) Math.pow(sdev,3));
			kurt = (float)kurt / (nc * (float) Math.pow(var, 2)) - 3;
		}
		q=0;
		pr=0;

		int[] hist = new int[(int)max+1];
		for(i=0;i<nc;i++){
			hist[data[i]]++; // load histogram
		}

		for (i=0;i<=max;i++){
			if (hist[i]> msf){
				msf=hist[i];
				mode=i;
			}
			if (hist[i]>0)
				pr+=((double)hist[i]/(double)nc)*(Math.log((double)hist[i]/(double)nc)/log2);

			q+=hist[i];
			if (q<=nco2)
				median=i; //simple median
		}
		if (min==max)
			median=max;// median in a single bin case

		if (pr<0)
			pr=-pr;

		pars[0]=totl;
		pars[1]= min;
		pars[2]= max;
		pars[3]= mode;
		pars[4]= median;
		pars[5]=ave;
		pars[6]=adev;
		pars[7]=sdev;
		pars[8]=var;
		pars[9]=skew;
		pars[10]=kurt;
		pars[11]=(float)pr;
	}


	void showAbout() {
		IJ.showMessage("About Phytoliths...",
		"Based on Particles8_ by Gabriel Landini,  G.Landini@bham.ac.uk\n"+
		"ImageJ plugin for estimating various statistics of 8-connected\n"+
		"particles.\n"+
		"WARNING!: This plugin does not return the same values as the original\n"+
		"ImageJ \'Analyze Particles\' command because it uses a different concept.\n"+
		"Here, \'Perimeter\' & \'Area\' are measured based on the polygon given by\n"+
		"the centres of the boundary pixels. \'Area\' disregards \'holes\' in the particles\n"+
		"(i.e. it returns the area inside the boundary), but \'Pixels\' does not.\n"+
		"CHull & CArea are also calculated from the centres of the pixels in the convex hull.\n"+
		"Note that 1 pixel particles have 0 area, etc.\n"+
		"Filtered particles are deleted from the image");
	}
	

//The following code for Breadth, Convex Perimeter, and Convex Area was written by AnnaLisa Maddox
//While employed as a research assistant to Dr. Terry Ball at Brigham Young University 2010-2014
//I would have used the convh function, but I couldn't understand G. Landini's code

	//using method of calipers
	//find minimum bounding box
	//smallest dimension is Breadth
	//largest dimension is Length
	//returns array containing breadth, length, and squareness.
	double[] Breadth(Vector vx, Vector vy)
	{
		double rotations = 360.0;
		double step = 360.0/rotations;
		double[] x = new double[vx.size()];
		double[] y = new double[vy.size()];

		double small_angle;
		double s_minx, s_maxx, s_miny, s_maxy;
		
		for(int j = 0; j<x.length;)//transfer vector into array
		{
			x[j] = (double)((Integer)(vx.get(j))).intValue();
			y[j] = (double)((Integer)(vy.get(j))).intValue();
			j++;
		}

		double x0 = x[0];
		double y0 = y[0];

		double dim1, dim2;
		double minDim = 0;
		double maxDim = 0;
		
		for(int i = 0; i<(int)rotations; i++)//rotate object by degree increments
		{
			double minX = x0;//if I reintialize these values for each rotation my minDim = 0;
			double maxX = x0;
			double minY = y0;
			double maxY = y0;

			for(int j = 0; j<x.length; j++)//determine bounding box at this rotation
			{
				if(x[j]<=minX)
					minX = x[j];
				if(x[j]>=maxX)
					maxX = x[j];
				if(y[j]<=minY)
					minY = y[j];
				if(y[j]>=maxY)
					maxY = y[j];
			}

			//determine smallest and largest dimensions at this rotation
			dim1 = maxX-minX;
			dim2 = maxY-minY;

			if(i == 0)//set initial minimum dimension
			{
				minDim = dim1;
				maxDim = dim1;
				//find center coordinates of feature
				x0 = (minX + maxX) * 0.5;
				y0 = (minY + maxY) * 0.5;
			}

			if(dim1<minDim)
				minDim = dim1;
			else if(dim1>maxDim)
				maxDim = dim1;
			if(dim2<minDim)
				minDim = dim2;
			else if(dim2>maxDim)
				maxDim = dim2;
			
			//determine smallest bounding box
			if(dim1*dim2<minDim*maxDim)
			{
				small_angle = step*(double)i;
				s_minx = minX;
				s_maxx = maxX;
				s_miny = minY;
				s_maxy = maxY;
			}
			
			//rotate one step about feature center
			for(int j = 0; j<x.length; j++)
			{
				// Use the original points as the starting point for each rotation to avoid roundoff errors.
				double x1 = (double)((Integer)(vx.get(j))).intValue();
				double y1 = (double)((Integer)(vy.get(j))).intValue();

				double rotationNumber = (double)i;
				double angle = step * rotationNumber;

				x[j] = (x1-x0)*Math.cos(angle) - (y1-y0)*Math.sin(angle)+x0;
				y[j] = (y1-y0)*Math.cos(angle) + (x1-x0)*Math.sin(angle)+y0;
			}
		}

		//account for measuring distances from pixel centers
		minDim++;
		maxDim++;
			//Find Smallest Bounding Rectangle (by area)
			//Store rotation angle
			//Store minx,maxx,miny,maxy
//		double S = 0;//squareness factor
//		double w1 = 0;
//		double w2 = 0;
//		double L;
//		//rotate pixels to smallest bounding rectangle
//		for(int j = 0; j<x.length; j++)
//		{
//			// Use the original points as the starting point for each rotation to avoid roundoff errors.
//			double x1 = (double)((Integer)(vx.get(j))).intValue();
//			double y1 = (double)((Integer)(vy.get(j))).intValue();
//
//			x[j] = (x1-x0)*Math.cos(small_angle) - (y1-y0)*Math.sin(small_angle)+x0;
//			y[j] = (y1-y0)*Math.cos(small_angle) + (x1-x0)*Math.sin(small_angle)+y0;
//		}
//		if(s_maxx-s_minx>s_maxy-s_miny)
//		{//length is horizontal
//			for(int i =0; i<(s_maxx-s_minx)/2; i++)
//			{//analyze each half of length for widest width
//				
//				//analyze for smallest length
//				
//			}
//		}
		
		//analyze each half of length for widest width
		//analyze for smallest length
		//return s=(w1+w2)/(2*L)
		
		double[] b_w = {minDim, maxDim};

		return b_w;		
	}
	
	/**
	 * This function attempts to intelligently decide how to measure fiber length using
	 * some hints from other measurements. Right now all it does is use one of Russ' formulas
	 * if the shape is circular. Otherwise, it uses the skeleton method.
	 * @author Ben Draut (drautb@gmail.com)
	 * @date 4/9/14
	 */
	private double IntelligentFiberLength(ImageProcessor ip, Vector vx, Vector vy, 
										  double length, double width, double perimeter, double area) {

		// Skeletons in circular shapes don't extend all the way to the edge, so it 
		// it looks like a circular object, just use it's longest measurement.
		double RECT_THRESHOLD = 20.0;
		double CIRCLE_THRESHOLD = 400.0;

		double approxRadius = (length + width) / 4.0;

		if (Math.abs(width - length) < RECT_THRESHOLD &&
			Math.abs((Math.PI * Math.pow(approxRadius, 2)) - area) < CIRCLE_THRESHOLD) {
			return (float)(0.5*perimeter) - 2*area/perimeter;
		}
			
		return SkeletonFiberLength(ip, vx, vy);
	}

	/**
	 * This function calculates fiber length by discovering the skeleton of the image,
	 * and then finding the longest path from one end of the skeleton to another.
	 * @author Ben Draut (drautb@gmail.com)
	 * @date 4/1/14
	 */
	private double SkeletonFiberLength(ImageProcessor ip, Vector vx, Vector vy) {
		int minX = ((Integer)vx.get(0)).intValue();
		int maxX = ((Integer)vx.get(0)).intValue();
		int minY = ((Integer)vy.get(0)).intValue();
		int maxY = ((Integer)vy.get(0)).intValue();

		for (int x=1; x<vx.size(); x++) {
			int thisX = ((Integer)vx.get(x)).intValue();
			if (thisX < minX) {
				minX = thisX;
			} else if (thisX > maxX) {
				maxX = thisX;
			}
		}

		for (int y=1; y<vy.size(); y++) {
			int thisY = ((Integer)vy.get(y)).intValue();
			if (thisY < minY) {
				minY = thisY;
			} else if (thisY > maxY) {
				maxY = thisY;
			}
		}

		// Now we have the min and max x and y values in each vector,
		// so now we'll build a bitset that represents the ROI.
		int width = maxX - minX + 2;
		int height = maxY - minY + 2;

		BitSet region = createBitSet(ip, vx, vy, minX, minY, width, height);
		BitSet skeleton = getSkeleton(region, width, height);

		return findLongestPath(skeleton, width, height);
	}

	private BitSet createBitSet(ImageProcessor ip, Vector vx, Vector vy, 
								int minX, int minY, int width, int height) {
		BitSet image = new BitSet(width*height);
		image.clear();

		for (int x=0; x<width; x++) {
			for (int y=0; y<height; y++) {
				boolean value = ip.get(minX + x, minY + y) == 0 ? false : true;
				image.set(y*width + x, value);
			}
		}

		return image;
	}

	private BitSet getSkeleton(BitSet region, int width, int height) {
		boolean deleted = false;

		do {
			deleted = skeletonIteration(region, width, height);
		} while (deleted);
		
		convertToMConnectivity(region, width, height);

		return region;
	}

	/**
	 * Does one iteration (a single pass with 8 different structuring elements)
	 * to skeletonize the bitset. (Gonzalez/Woods pg. 800?)
	 */
	private boolean skeletonIteration(BitSet region, int width, int height) {
		boolean deletedPoint = false;
		BitSet mask = (BitSet)region.clone();

		for (int b=1; b<=8; b++) {
			
			mask.clear();
			int hashCode = mask.hashCode();

			for (int x=0; x<width; x++) {
				for (int y=0; y<height; y++) {
					if (!getBit(region, x, y, width, height)) {
						continue;
					}

					boolean p1 = getBit(region, x-1, y-1, width, height);
					boolean p2 = getBit(region, x, y-1, width, height);
					boolean p3 = getBit(region, x+1, y-1, width, height);
					boolean p4 = getBit(region, x-1, y, width, height);
					boolean p5 = getBit(region, x+1, y, width, height);
					boolean p6 = getBit(region, x-1, y+1, width, height);
					boolean p7 = getBit(region, x, y+1, width, height);
					boolean p8 = getBit(region, x+1, y+1, width, height);

					boolean test = false;
					switch (b) {
						case 1:
							test = !p1 && !p2 && !p3 && p6 && p7 && p8;
							break;

						case 2:
							test = !p2 && !p3 && p4 && !p5 && p6 && p7;
							break;

						case 3:
							test = p1 && !p3 && p4 && !p5 && p6 && !p8;
							break;

						case 4:
							test = p1 && p2 && p4 && !p5 && !p7 && !p8;
							break;

						case 5:
							test = p1 && p2 && p3 && !p6 && !p7 && !p8;
							break;

						case 6:
							test = p2 && p3 && !p4 && p5 && !p6 && !p7;
							break;

						case 7:
							test = !p1 && p3 && !p4 && p5 && !p6 && p8;
							break; 

						case 8:
							test = !p1 && !p2 && !p4 && p5 && p7 && p8;
							break;

						default:
							break;
					}

					if (test) {
						mask.set(y*width + x, true);
					}
				}
			}

			deletedPoint = deletedPoint || (mask.hashCode() != hashCode);

			region.xor(mask);
		}

		return deletedPoint;
	}

	private void convertToMConnectivity(BitSet region, int width, int height) {	
		for (int x=0; x<width; x++) {
			for (int y=0; y<height; y++) {
				
				if (!getBit(region, x, y, width, height)) {
						continue;
				}

				boolean p1 = getBit(region, x-1, y-1, width, height);
				boolean p2 = getBit(region, x, y-1, width, height);
				boolean p3 = getBit(region, x+1, y-1, width, height);
				boolean p4 = getBit(region, x-1, y, width, height);
				boolean p5 = getBit(region, x+1, y, width, height);
				boolean p6 = getBit(region, x-1, y+1, width, height);
				boolean p7 = getBit(region, x, y+1, width, height);
				boolean p8 = getBit(region, x+1, y+1, width, height);

				if ((p2 && p5 && !p6) || 
					(p5 && p7 && !p1) || 
					(p7 && p4 && !p3) || 
					(p4 && p2 && !p8)) {
					region.set(getBitIdx(x, y, width), false);
				}
			}
		}
	}

	private static final double ADJACENT = 1.0;
	private static final double DIAGONAL = 1.41421356237;

	private double findLongestPath(BitSet skeleton, int width, int height) {
		// This finds each endpoint of the skeleton, and finds the longest path
		// starting at that point. It returns the longest path it found.		
		ArrayList<Double> lengths = new ArrayList<Double>();

		for (int x=0; x<width; x++) {
			for (int y=0; y<height; y++) {
				if (countNeighbors(skeleton, x, y, width, height) == 1) {
					// Mark starting point as visited
					BitSet visited = (BitSet)skeleton.clone();
					visited.clear();
					visited.set(y*width + x, true);
					
					// Start recursion on each side
					double length = Math.max(ADJACENT + recFindLongestPath(skeleton, visited, x + 1, y, width, height),
									Math.max(ADJACENT + recFindLongestPath(skeleton, visited, x - 1, y, width, height),
									Math.max(ADJACENT + recFindLongestPath(skeleton, visited, x, y + 1, width, height),
									Math.max(ADJACENT + recFindLongestPath(skeleton, visited, x, y - 1, width, height),
									Math.max(DIAGONAL + recFindLongestPath(skeleton, visited, x + 1, y + 1, width, height),
									Math.max(DIAGONAL + recFindLongestPath(skeleton, visited, x - 1, y + 1, width, height),
									Math.max(DIAGONAL + recFindLongestPath(skeleton, visited, x + 1, y - 1, width, height),
											 DIAGONAL + recFindLongestPath(skeleton, visited, x - 1, y - 1, width, height))))))));

					lengths.add(length);					
				}
			}
		}		

		// System.out.println("LENGTHS: " + lengths);

		return Collections.max(lengths).doubleValue();
	}

	private double recFindLongestPath(BitSet image, BitSet visited, int x, int y, int width, int height) {
		if (x < 0 || y < 0 || x > width - 1 || y > height - 1) {
			return 0.0;
		}

		int idx = y*width + x;

		if (visited.get(idx)) {
			return 0.0;
		} else {
			visited.set(idx, true);
		}

		if (!image.get(idx)) {
			return 0.0;
		}

		return Math.max(ADJACENT + recFindLongestPath(image, visited, x + 1, y, width, height),
			   Math.max(ADJACENT + recFindLongestPath(image, visited, x - 1, y, width, height),
			   Math.max(ADJACENT + recFindLongestPath(image, visited, x, y + 1, width, height),
			   Math.max(ADJACENT + recFindLongestPath(image, visited, x, y - 1, width, height),
			   Math.max(DIAGONAL + recFindLongestPath(image, visited, x + 1, y + 1, width, height),
			   Math.max(DIAGONAL + recFindLongestPath(image, visited, x - 1, y + 1, width, height),
			   Math.max(DIAGONAL + recFindLongestPath(image, visited, x + 1, y - 1, width, height),
						DIAGONAL + recFindLongestPath(image, visited, x - 1, y - 1, width, height))))))));
	}

	private int countNeighbors(BitSet bits, int x, int y, int width, int height) {
		int count = 0;
		if (!getBit(bits, x, y, width, height)) {
			return count;
		}

		count += getBit(bits, x + 1, y, width, height) ? 1 : 0;
		count += getBit(bits, x + 1, y + 1, width, height) ? 1 : 0;
		count += getBit(bits, x, y + 1, width, height) ? 1 : 0;
		count += getBit(bits, x - 1, y + 1, width, height) ? 1 : 0;
		count += getBit(bits, x - 1 , y, width, height) ? 1 : 0;
		count += getBit(bits, x - 1, y - 1, width, height) ? 1 : 0;
		count += getBit(bits, x, y - 1, width, height) ? 1 : 0;
		count += getBit(bits, x + 1, y - 1, width, height) ? 1 : 0;

		return count;
	}

	private boolean getBit(BitSet bits, int x, int y, int width, int height) {
		if (x < 0 || y < 0 || x > width || y > height)
			return false;

		return bits.get(getBitIdx(x, y, width));
	}

	private int getBitIdx(int x, int y, int width) {
		return y*width + x;
	}
	
	//using calipers method
	//calculate convex perimeter by rotating through 360 degrees
	//at each rotation, find min/max x,y values - the bounding box
	//store each pixel in list edge
	//compare edge with vx, vy (perimeter pixels)
	//arrange convex perimeter points in order
	void Conv_Perim(Vector vx, Vector vy)
	{
		List edge = new List();
		convP.clear();
		int rotations = 360;
		double step = 360/rotations;
		//arrays xx and yy contain original perimeter pixel values
		//arrays x and y contain rotated perimeter pixel values
		int[] xx = new int[vx.size()];
		int[] yy = new int[vy.size()];
		double[] x = new double[vx.size()];
		double[] y = new double[vy.size()];
		for(int j = 0; j<xx.length;)//transfer vectors into arrays
		{
			xx[j] = ((Integer)(vx.get(j))).intValue();
			yy[j] = ((Integer)(vy.get(j))).intValue();
			x[j] = (double)((Integer)(vx.get(j))).intValue();
			y[j] = (double)((Integer)(vy.get(j))).intValue();
			j++;
		}
		double minX = x[0];
		double maxX = x[0];
		double minY = y[0];
		double maxY = y[0];
		double perimeter = 0;
		Node cur = new Node();
		double x0 = 0;
		double y0 = 0;
		
		//analyze image through 360 degrees
		for(int i = 0; i<rotations; i++)
		{
			minX = x[0];
			maxX = x[0];
			minY = y[0];
			maxY = y[0];
			//nodes to keep track of the min/max x,y coordinates
			Node minx = new Node(xx[0],yy[0]);
			Node maxx = new Node(xx[0],yy[0]);
			Node miny = new Node(xx[0],yy[0]);
			Node maxy = new Node(xx[0],yy[0]);
			int minx_x= xx[0];
			int maxx_x = xx[0];
			int miny_x= xx[0];
			int maxy_x = xx[0];
			int minx_y= yy[0];
			int maxx_y = yy[0];
			int miny_y= yy[0];
			int maxy_y = yy[0];
			//find min/max x,y values using rotated coordinates
			//store original coordinates (of each determined edge point) in min/max nodes
			for(int j = 0; j<xx.length; j++)
			{
				if(x[j]<=minX)//compare rotated coordinates to current min/max value
				{
					minX = x[j];
					minx.x = xx[j];
					minx.y = yy[j];
					minx_x = xx[j];
					minx_y = yy[j];
				}
				else if(x[j]>=maxX)
				{
					maxX = x[j];
					maxx.x = xx[j];
					maxx.y = yy[j];
					maxx_x = xx[j];
					maxx_y = yy[j];
				}
				if(y[j]<=minY)
				{
					minY = y[j];
					miny.x = xx[j];
					miny.y = yy[j];
					miny_x = xx[j];
					miny_y = yy[j];
				}
				else if(y[j]>=maxY)
				{
					maxY = y[j];
					maxy.x = xx[j];
					maxy.y = yy[j];
					maxy_x = xx[j];
					maxy_y = yy[j];
				}
			}
			if(i==0)//determine center of bounding box
			{
				x0 = (maxX-minX)*.5+maxX;
				y0 = (maxY-minY)*.5+maxY;
			}
			//rotate each pixel around object center
			for(int j = 0; j<xx.length; j++)
			{
				double x1 = x[j];
				double y1 = y[j];
				x[j] = (x1-x0)*Math.cos(step) - (y1-y0)*Math.sin(step)+x0;
				y[j] = (y1-y0)*Math.sin(step) + (x1-x0)*Math.cos(step)+y0;
			}
			//store edge coordinates in linked list
			edge.insertFront(minx_x, minx_y);
			edge.insertFront(maxx_x, maxx_y);
			edge.insertFront(miny_x, miny_y);
			edge.insertFront(maxy_x, maxy_y);
		}
		
		//compare edge pixels with perimeter to get rid of false convex perimeter points
		for(int i = 0; i<xx.length; i++)
		{
			cur = edge.start();
			boolean cont = true;
			while(cur!=null && cont == true)
			{
				if(xx[i]==cur.x && yy[i] == cur.y)//perimeter pixels match
				{
					convP.insertFront(xx[i], yy[i]);
					cont= false;
				}
				cur=cur.Next;
			}
		}
		cur=convP.start();
		boolean straightx = true;
		boolean straighty = true;
	}
	
	private static class Node
	{
		public int x;
		public int y;
		public int cc;//chain code
		public Node Next;
		
		public Node()
		{
			x = 0;
			y = 0;
			Next = null;
		}
		public Node(int x_coord, int y_coord)
		{
			x = x_coord;
			y = y_coord;
			cc = -1;
		}
		public Node(int x_coord, int y_coord, int c)
		{
			x = x_coord;
			y = y_coord;
			cc = c;
		}
	}
	
	private static class List
	{
		private Node list;
		public int size;
		private Node xmin;
		private Node xmax;
		private Node ymin;
		private Node ymax;
		
		public List()
		{
			size = 0;
			list = null;
		}
		
		public void clear()
		{
			list = null;
			size = 0;
		}
		
		public Node start()
		{
			return list;
		}
		
		public Node last()
		{
			Node cur = list;
			int n = 0;
			while(n<size-1&&cur.Next!=null)
			{
				cur = cur.Next;
				n++;
			}
			return cur;
		}
		
		public int getx(int n)
		{
			if(n>size||size==0)
				return 0;
			Node cur = list;
			for(int i = 0; i<n; i++)
			{
				if(cur.Next!= null)
					cur = cur.Next;
				else
					return 0;
			}
			return cur.x;
		}
		
		public int gety(int n)
		{
			if(n>size||size==0)
				return 0;
			Node cur = list;
			for(int i = 0; i<n; i++)
			{
				if(cur.Next!= null)
					cur = cur.Next;
				else
					return 0;
			}
			return cur.y;
		}
		
		public boolean find(int x, int y)
		{
			for(Node cur = list; cur!= null;)//check entire list for duplicate pixel
			{
				if(cur.x == x && cur.y == y)//This is a repeat pixel, don't insert it
				{
					return true;
				}
				cur = cur.Next;
			}
			return false;
		}
		
		public void insertFront(int xx, int yy)
		{
			if(find(xx,yy)==false)
			{		
				//insert at beginning
				Node n = new Node(xx,yy);
				n.Next = list;
				list = n;
				if(size == 0)
				{
					xmin = n;
					xmax = n;
					ymin = n;
					ymax = n;
				}
				else if(n.x<xmin.x)
				{
					xmin = n;
				}
				else if(n.x>xmax.x)
				{
					xmax = n;
				}
				else if(n.y<ymin.y)
				{
					ymin = n;
				}
				else if(n.y>ymax.y)
				{
					ymax = n;
				}
				++size;
			}
		}
		
		public Node xMin()
		{
			return xmin;
		}
		
		public Node xMax()
		{
			return xmax;
		}
		
		public Node yMin()
		{
			return ymin;
		}
		
		public Node yMax()
		{
			return ymax;
		}
	}
		
	
    //Calculates convex area by pixel edges (found from the bounding box)
	//And the equation for polygon area from John C. Russ's book "Computer-Assisted Microscopy" pg. 183
	double Conv_Area()
	{
		int minx=convP.start().x;
		int maxx=convP.start().x;
		int miny=convP.start().y;
		int maxy=convP.start().y;
		double[] x = new double[convP.size*2];//max number of x points possible
		double[] y = new double[convP.size*2];//max number of y points possible
		double carea = 0;
		Side side1, side2;
		Node last = convP.last();
		
		if(convP.start()==null)
			return 0;
		int number=0;
		
		if(convP.start()!=null)
		{
			Node cur = convP.start();
			for(int i = 0; i<convP.size; i++)
			{
				double xdd1 = cur.x;
				double ydd1 = cur.y;
				cur=cur.Next;
			}
		}
	   
		//find min/max x,y values
		for(Node cur = convP.start(); number<convP.size; cur=cur.Next)
		{
			if(cur.x<=minx)
			{
				minx = cur.x;
			}
			else if(cur.x>=maxx)
			{
				maxx = cur.x;
			}
			if(cur.y<=miny)
			{
				miny = cur.y;
			}
			else if(cur.y>=maxy)
			{
				maxy = cur.y;
			}
			number++;
		}

		int n= 0;
		int arrayLength=convP.size;
		Node next;
		//for each convP pixel, change coordinates from pixel centers to pixel edges 
		for(Node cur = convP.start(); n<arrayLength; cur=cur.Next)
		{
			if(cur.Next==null)
				next=convP.last();
			else
				next=cur.Next;
			//update edges
			//ccw rotation
			if(last.x==cur.x&&last.y==cur.y){//same pixel - do nothing
			}
			else if(last.x!=cur.x&&cur.x!=next.x&&last.y!=cur.y&&cur.y!=next.y)//pivot pixel - needs two corner points
			{
				arrayLength++;
				if(last.y<cur.y&&last.x>cur.x)//moving down
				{
					x[n]=cur.x-0.5;
					y[n]=cur.y-0.5;
					n++;
					x[n]=cur.x-0.5;
					y[n]=cur.y+0.5;
				}
				else if(last.y<cur.y&&last.x<cur.x)//moving right
				{
					x[n]=cur.x-0.5;
					y[n]=cur.y+0.5;
					n++;			
					x[n]=cur.x+0.5;
					y[n]=cur.y+0.5;
				}
				else if(last.y>cur.y&&last.x<cur.x)//moving up
				{
					x[n]=cur.x+0.5;
					y[n]=cur.y+0.5;
					n++;	
					x[n]=cur.x+0.5;
					y[n]=cur.y-0.5;
				}	
				else if(last.y>cur.y&&last.x>cur.x)//moving left 
				{
					x[n]=cur.x+0.5;
					y[n]=cur.y-0.5;
					n++;		
					x[n]=cur.x-0.5;
					y[n]=cur.y-0.5;
				}
			}
			else if(last.x<cur.x)//moving right
			{
				if(last.y>=cur.y){//moving up
					x[n]=cur.x+0.5;
					y[n]=cur.y+0.5;
				}
				else //if(last.y<cur.y)
				{//moving down
					x[n]=cur.x-0.5;
					y[n]=cur.y+0.5;
				}
			}
			else if(last.x>cur.x)//moving left
			{
				if(last.y>cur.y){//moving up
					x[n]=cur.x+0.5;
					y[n]=cur.y-0.5;
				}
				else// if(last.y<=cur.y){//moving down
				{
					x[n]=cur.x-0.5;
					y[n]=cur.y-0.5;
				}
			}
			else	// last.x==cur.x && last.y!=cur.y aka moving up or down
			{
				if(last.y>cur.y)//moving up
				{
					x[n]=cur.x+0.5;
					y[n]=cur.y-0.5;
				}
				else //moving down
				{
					x[n]=cur.x-0.5;
					y[n]=cur.y+0.5;
				}
			}
			last = cur;
			n++;
		}

		//Area Equation
		double x1 = 0;
		double x2 = 0;
		double y1 = 0;
		double y2 = 0;

		int i=0;
		double subtract =0;
		for(; i<arrayLength; i++)
		{
			if(i==arrayLength-1)
			{
				x2=x[0];
				y2=y[0];
			}
			else
			{
				x2=x[i+1];
				y2=y[i+1];
			}
			x1 = x[i];
			y1 = y[i];
			carea += (x2*y1)-(x1*y2);
			//subtract some area based on slope (to get more accurate pixel count)
			double xp=Math.abs(x1-x2);
			double yp=Math.abs(y1-y2);
			double m = yp/xp;//slope
			if(m>999)
				m=0;
			else if(m>0&&m<1)
				subtract+=(yp-0.5)/(m*2);
			else if(m>1&&m<999)
				subtract+=m/2*(xp-0.5);
			else if(m==1)
				subtract+=m/2*(xp);
		}
		carea=carea/2;
		carea-=subtract;
		if(carea<0)
			carea=-1;
		return carea;			
	}
	
}





